package uebung3;

import java.io.*;
import java.util.*;

import com.sun.javafx.runtime.Entry;
import javafx.async.RunnableFuture;

public class JavaFileMonitor implements RunnableFuture {
    FileMonitorListener listener;
    File file;
    HashMap<File,Long> files;
    long interval = 1000;

    public JavaFileMonitor(File file, FileMonitorListener listener) {
        assert(file.exists() && file.isDirectory());
        this.listener = listener;
        this.file = file;
        this.files = new HashMap<File,Long>();
    }

    public void run() throws Exception {
        while(true) {
            List<File> current = Arrays.asList(file.listFiles());
            for(File f: current) {
                if(files.containsKey(f)) {
                    if(files.get(f)<f.lastModified()) {
                        postFileChanged(f);
                        files.put(f,new Long(f.lastModified()));
                    }
                } else {
                    files.put(f,new Long(f.lastModified()));
                    postFileAdded(f);
                }
            }
            Set<File> obsolete = new HashSet<File>(files.keySet());
            obsolete.removeAll(current);
            for(File f: obsolete) {
                files.remove(f);
                postFileRemoved(f);
            }
            Thread.sleep(interval);
        }
    }

    public void postFileAdded(final File file) {
        Entry.deferAction(new Runnable() {
            public void run() {
                listener.fileAdded(file);
            }
        });
    }

    public void postFileChanged(final File file) {
        Entry.deferAction(new Runnable() {
            public void run() {
                listener.fileChanged(file);
            }
        });
    }

    public void postFileRemoved(final File file) {
        Entry.deferAction(new Runnable() {
            public void run() {
                listener.fileRemoved(file);
            }
        });
    }
}